@@ -14,9 +14,8 @@
 #    under the License.
 
 import abc
-import six
 
-from midonet.neutron._i18n import _LE
+import six
 
 from oslo_log import log as logging
 from oslo_utils import excutils
@@ -32,7 +31,7 @@ def load_client(conf):
         return client
     except ImportError:
         with excutils.save_and_reraise_exception():
-            LOG.exception(_LE("Error loading midonet client '%(client)s'"),
+            LOG.exception("Error loading midonet client '%(client)s'",
                           {'client': conf.client})
 
 
@@ -207,14 +206,6 @@ class MidonetClientBase(object):
     def delete_agent_membership_postcommit(self, agent_membership_id):
         pass
 
-    # Agent extension
-
-    def get_agent(self, agent_id):
-        return None
-
-    def get_agents(self):
-        return []
-
     # LBaaS methods - these are not inside a DB transaction.  DB updates as
     # well as blocking API calls are allowed.
 
@@ -280,8 +271,8 @@ class MidonetClientBase(object):
     def create_ipsec_site_conn(self, context, ipsec_site_conn):
         pass
 
-    def update_ipsec_site_conn(self, context, ipsec_site_conn_id,
-            ipsec_site_conn):
+    def update_ipsec_site_conn(
+            self, context, ipsec_site_conn_id, ipsec_site_conn):
         pass
 
     def delete_ipsec_site_conn(self, context, ipsec_site_conn_id):
@@ -414,3 +405,9 @@ class MidonetClientBase(object):
 
     def delete_qos_policy(self, context, policy):
         pass
+
+    # MidoNet Pool Member
+    def get_pool_member(self, context, member_id):
+        pass
+
+    # REVISIT(yamamoto): LBaaS v2
