@@ -13,11 +13,11 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from neutron_lib.db import api as db_api
+
 from midonet.neutron.client import base
 from midonet.neutron.db import task_db as task
 
-import neutron.db.api as db
-
 
 class MidonetClusterClient(base.MidonetClientBase):
 
@@ -25,7 +25,7 @@ class MidonetClusterClient(base.MidonetClientBase):
         self.conf = conf
 
     def initialize(self):
-        task.create_config_task(db.get_session(), dict(self.conf))
+        task.create_config_task(db_api.get_writer_session(), dict(self.conf))
 
     def create_network_precommit(self, context, network):
         task.create_task(context, task.CREATE, data_type=task.NETWORK,
@@ -159,4 +159,13 @@ class MidonetClusterClient(base.MidonetClientBase):
         task.create_task(context, task.DELETE, data_type=task.HEALTH_MONITOR,
                          resource_id=health_monitor_id)
 
-    # REVISIT(yamamoto): recent additions, including QoS, lbaasv2
+    # TODO(yamamoto): Implement the following
+    # - FWaaS v1
+    # - VPNaaS
+    # - L2 gateway
+    # - gateway device
+    # - BGP
+    # - logging resource / firewall log
+    # - Tap as a service
+    # - QoS
+    # - LBaaS v2
