@@ -14,16 +14,15 @@
 #    under the License.
 
 from neutron_lib.api import validators
-from neutron_lib import constants
+from neutron_lib.callbacks import events
+from neutron_lib.callbacks import registry
+from neutron_lib.callbacks import resources
 from neutron_lib.db import model_base
+from neutron_lib.db import utils as db_utils
 from neutron_lib import exceptions as n_exc
+from neutron_lib.exceptions import l3 as l3_exc
+from neutron_lib.plugins import constants
 from neutron_lib.plugins import directory
-
-from neutron.callbacks import events
-from neutron.callbacks import registry
-from neutron.callbacks import resources
-from neutron.db import common_db_mixin
-from neutron.extensions import l3
 from oslo_db import exception as db_exc
 from oslo_utils import uuidutils
 import sqlalchemy as sa
@@ -31,7 +30,8 @@ from sqlalchemy.ext import declarative
 from sqlalchemy import orm
 from sqlalchemy.orm import exc
 
-from midonet.neutron.common import constants as midonet_const
+from neutron.db import common_db_mixin
+
 from midonet.neutron.extensions import gateway_device as gw_device_ext
 
 GATEWAY_DEVICES = 'midonet_gateway_devices'
@@ -98,9 +98,9 @@ def _resource_id_column(foreign_key):
 def _gateway_device_relation(class_name, ref_key):
     relation = "GatewayDevice.id==" + class_name + ".device_id"
     return orm.relationship(
-                GatewayDevice,
-                backref=orm.backref(ref_key, cascade='delete', lazy='joined'),
-                primaryjoin=relation)
+        GatewayDevice,
+        backref=orm.backref(ref_key, cascade='delete', lazy='joined'),
+        primaryjoin=relation)
 
 
 class GatewayOverlayRouterDevice(GatewayVirtualDevice, model_base.BASEV2):
@@ -157,6 +157,7 @@ class GatewayRemoteMacTable(model_base.BASEV2):
         primaryjoin="GatewayDevice.id==GatewayRemoteMacTable.device_id")
 
 
+@registry.has_registry_receivers
 class GwDeviceDbMixin(gw_device_ext.GwDevicePluginBase,
                       common_db_mixin.CommonDbMixin):
     """Mixin class to add gateway device."""
@@ -210,10 +211,13 @@ class GwDeviceDbMixin(gw_device_ext.GwDevicePluginBase,
         else:
             return gw_hw_vtep_db
 
-    def _get_remote_mac_entry(self, context, id):
+    def _get_remote_mac_entry(self, context, id, gateway_device_id):
         try:
             query = self._model_query(context, GatewayRemoteMacTable)
             rmt_db = query.filter(GatewayRemoteMacTable.id == id).one()
+            if rmt_db.device_id != gateway_device_id:
+                raise gw_device_ext.RemoteMacEntryWrongGatewayDevice(
+                    id=id, gateway_device_id=gateway_device_id)
 
         except exc.NoResultFound:
             raise gw_device_ext.RemoteMacEntryNotFound(id=id)
@@ -258,8 +262,8 @@ class GwDeviceDbMixin(gw_device_ext.GwDevicePluginBase,
             res['management_port'] = None
             res['management_protocol'] = None
             res['resource_id'] = gw_dev_db.overlay_router[0]['resource_id']
-        res['tunnel_ips'] = list(map(lambda n: n['tunnel_ip'],
-                gw_dev_db.tunnel_ip_list))
+        res['tunnel_ips'] = list(map(
+            lambda n: n['tunnel_ip'], gw_dev_db.tunnel_ip_list))
         for item in gw_dev_db.mac_table_list:
             entry = {'id': item['id'],
                      'mac_address': item['mac_address'],
@@ -291,16 +295,17 @@ class GwDeviceDbMixin(gw_device_ext.GwDevicePluginBase,
 
     def _update_gateway_device_db(self, context, gw_dev_id, data):
         """Update the DB object.
-           following parameter can be updated.
-           name, tunnel_ips
+
+        following parameter can be updated.
+            name, tunnel_ips
         """
 
         add_ips, delete_ips = [], []
         with context.session.begin(subtransactions=True):
             gw_dev_db = self._get_gateway_device(context, gw_dev_id)
             if data.get('tunnel_ips'):
-                exist_ips = list(map(lambda n: n['tunnel_ip'],
-                            gw_dev_db.tunnel_ip_list))
+                exist_ips = list(map(
+                    lambda n: n['tunnel_ip'], gw_dev_db.tunnel_ip_list))
                 add_ips = set(data['tunnel_ips']) - set(exist_ips)
                 delete_ips = set(exist_ips) - set(data['tunnel_ips'])
             if delete_ips:
@@ -325,22 +330,22 @@ class GwDeviceDbMixin(gw_device_ext.GwDevicePluginBase,
         if not gw_dev['management_ip'] or not gw_dev['management_port']:
             raise gw_device_ext.HwVtepTypeInvalid(type=gw_dev['type'])
         if self._get_hw_vtep_from_management_ip(
-            context, gw_dev['management_ip']):
+                context, gw_dev['management_ip']):
             raise gw_device_ext.GatewayDeviceParamDuplicate(
                 param_name='management_ip',
                 param_value=gw_dev['management_ip'])
 
     def _validate_resource_router_vtep(self, context, router_id):
-        # Check specified router existance
+        # Check specified router existence
         l3plugin = directory.get_plugin(constants.L3)
 
         try:
             l3plugin.get_router(context, router_id)
-        except l3.RouterNotFound:
+        except l3_exc.RouterNotFound:
             raise gw_device_ext.ResourceNotFound(resource_id=router_id)
 
         if self._get_gateway_device_from_resource(
-               context, gw_device_ext.ROUTER_DEVICE_TYPE, router_id):
+                context, gw_device_ext.ROUTER_DEVICE_TYPE, router_id):
             raise gw_device_ext.DeviceInUseByGatewayDevice(
                 resource_id=router_id, resource_type='router')
 
@@ -351,7 +356,7 @@ class GwDeviceDbMixin(gw_device_ext.GwDevicePluginBase,
                                             gw_dev['resource_id'])
 
     def _validate_resource_vlan_network(self, context, network_id):
-        # Check specified netowrk existance
+        # Check specified netowrk existence
         core_plugin = directory.get_plugin()
 
         try:
@@ -360,7 +365,7 @@ class GwDeviceDbMixin(gw_device_ext.GwDevicePluginBase,
             raise gw_device_ext.ResourceNotFound(resource_id=network_id)
 
         if self._get_gateway_device_from_resource(
-               context, gw_device_ext.NETWORK_VLAN_TYPE, network_id):
+                context, gw_device_ext.NETWORK_VLAN_TYPE, network_id):
             raise gw_device_ext.DeviceInUseByGatewayDevice(
                 resource_id=network_id, resource_type='network')
 
@@ -386,7 +391,8 @@ class GwDeviceDbMixin(gw_device_ext.GwDevicePluginBase,
                                       gw_dev['type'])
 
         with context.session.begin(subtransactions=True):
-            gw_dev_db = GatewayDevice(id=uuidutils.generate_uuid(),
+            gw_dev_db = GatewayDevice(
+                id=uuidutils.generate_uuid(),
                 name=gw_dev['name'],
                 type=(gw_dev['type'] or gw_device_ext.HW_VTEP_TYPE),
                 tenant_id=tenant_id)
@@ -441,7 +447,7 @@ class GwDeviceDbMixin(gw_device_ext.GwDevicePluginBase,
 
     def delete_gateway_device_remote_mac_entry(self, context, id,
                                                gateway_device_id):
-        rmt_db = self._get_remote_mac_entry(context, id)
+        rmt_db = self._get_remote_mac_entry(context, id, gateway_device_id)
         with context.session.begin(subtransactions=True):
             context.session.delete(rmt_db)
 
@@ -452,8 +458,8 @@ class GwDeviceDbMixin(gw_device_ext.GwDevicePluginBase,
     def get_gateway_devices(self, context, filters=None, fields=None,
                             sorts=None, limit=None, marker=None,
                             page_reverse=False):
-        marker_obj = self._get_marker_obj(context, 'gateway_device', limit,
-                                          marker)
+        marker_obj = db_utils.get_marker_obj(self, context, 'gateway_device',
+                                             limit, marker)
 
         return self._get_collection(context,
                                     GatewayDevice,
@@ -467,8 +473,9 @@ class GwDeviceDbMixin(gw_device_ext.GwDevicePluginBase,
                                               filters=None, fields=None,
                                               sorts=None, limit=None,
                                               marker=None, page_reverse=False):
-        marker_obj = self._get_marker_obj(context, 'remote_mac_entry', limit,
-                                          marker)
+        marker_obj = db_utils.get_marker_obj(self, context, 'remote_mac_entry',
+                                             limit, marker)
+        filters['device_id'] = [gateway_device_id]
         return self._get_collection(context,
                                     GatewayRemoteMacTable,
                                     self._make_remote_mac_dict,
@@ -479,7 +486,7 @@ class GwDeviceDbMixin(gw_device_ext.GwDevicePluginBase,
 
     def get_gateway_device_remote_mac_entry(self, context, id,
                                             gateway_device_id, fields=None):
-        rme = self._get_remote_mac_entry(context, id)
+        rme = self._get_remote_mac_entry(context, id, gateway_device_id)
         return self._make_remote_mac_dict(rme, fields)
 
     def update_gateway_device(self, context, id, gw_device):
@@ -493,36 +500,35 @@ class GwDeviceDbMixin(gw_device_ext.GwDevicePluginBase,
         gw_dev_db = self._update_gateway_device_db(context, id, gw_dev)
         return self._make_gateway_device_dict(gw_dev_db)
 
-
-def gateway_device_callback(resource, event, trigger, **kwargs):
-    if resource == resources.ROUTER:
-        resource_id = kwargs['router_id']
-        gw_dev_type = gw_device_ext.ROUTER_DEVICE_TYPE
-        resource_type = 'router'
-    elif resource == midonet_const.MIDONET_NETWORK:
-        resource_id = kwargs['network_id']
-        gw_dev_type = gw_device_ext.NETWORK_VLAN_TYPE
-        resource_type = 'network'
-    else:
-        return
-    gw_dev_plugin = directory.get_plugin(midonet_const.GATEWAY_DEVICE)
-    if gw_dev_plugin:
-        context = kwargs.get('context')
-        if gw_dev_plugin._get_gateway_device_from_resource(context,
-                                                           gw_dev_type,
-                                                           resource_id):
+    @registry.receives(resources.ROUTER, [events.BEFORE_DELETE])
+    def _gateway_router_device_callback(self, resource, event,
+                                        trigger, payload=None):
+        # TODO(boden): refactor this back into _gateway_device_callback once
+        # NETWORK resources use paylaods
+        if resource == resources.ROUTER:
+            resource_id = payload.resource_id
+            gw_dev_type = gw_device_ext.ROUTER_DEVICE_TYPE
+            resource_type = 'router'
+        else:
+            return
+        context = payload.context
+        if self._get_gateway_device_from_resource(context,
+                                                  gw_dev_type,
+                                                  resource_id):
             raise gw_device_ext.DeviceInUseByGatewayDevice(
                 resource_id=resource_id, resource_type=resource_type)
 
-
-def subscribe():
-    registry.subscribe(
-        gateway_device_callback, resources.ROUTER, events.BEFORE_DELETE)
-
-    # We can only add PRECOMMIT_DELETE events in MidoNet ML2 mechanism driver
-    # while we can add both BEFORE_DELETE and PRECOMMIT_DELETE in MidoNet
-    # plugin. To keep the ML2 and plugin behavior the same, use
-    # PRECOMMIT_DELETE event for both ML2 and plugin.
-    registry.subscribe(
-        gateway_device_callback, midonet_const.MIDONET_NETWORK,
-        events.PRECOMMIT_DELETE)
+    @registry.receives(resources.NETWORK, [events.PRECOMMIT_DELETE])
+    def _gateway_device_callback(self, resource, event, trigger, **kwargs):
+        if resource == resources.NETWORK:
+            resource_id = kwargs['network_id']
+            gw_dev_type = gw_device_ext.NETWORK_VLAN_TYPE
+            resource_type = 'network'
+        else:
+            return
+        context = kwargs.get('context')
+        if self._get_gateway_device_from_resource(context,
+                                                  gw_dev_type,
+                                                  resource_id):
+            raise gw_device_ext.DeviceInUseByGatewayDevice(
+                resource_id=resource_id, resource_type=resource_type)
